searchNodes=[{"doc":"FunWithFlags, the Elixir feature flag library. This module provides the public interface to the library and its API is made of three simple methods to enable, disable and query feature flags. In their simplest form, flags can be toggled on and off globally. More advanced rules or &quot;gates&quot; are available, and they can be set and queried for any term that implements these protocols: The FunWithFlags.Actor protocol can be implemented for types and structs that should have specific rules. For example, in web applications it&#39;s common to use a %User{} struct or equivalent as an actor, or perhaps the current country of the request. The FunWithFlags.Group protocol can be implemented for types and structs that should belong to groups for which one wants to enable and disable some flags. For example, one could implement the protocol for a %User{} struct to identify administrators. See the Usage notes for a more detailed explanation.","ref":"FunWithFlags.html","title":"FunWithFlags","type":"module"},{"doc":"Returns a list of all flag names currently configured, as atoms. This can be useful for debugging or for display purposes, but it&#39;s not meant to be used at runtime. Undefined flags, for example, will be considered disabled.","ref":"FunWithFlags.html#all_flag_names/0","title":"FunWithFlags.all_flag_names/0","type":"function"},{"doc":"Returns a list of all the flags currently configured, as data structures. This function is provided for debugging and to build more complex functionality (e.g. it&#39;s used in the web GUI), but it is not meant to be used at runtime to check if a flag is enabled. To query the value of a flag, please use the enabled?2 function instead.","ref":"FunWithFlags.html#all_flags/0","title":"FunWithFlags.all_flags/0","type":"function"},{"doc":"Clears the data of a feature flag. Clears the data for an entire feature flag or for a specific Actor or Group gate. Clearing a boolean gate is not supported because a missing boolean gate is equivalent to a disabled boolean gate. Sometimes enabling or disabling a gate is not what you want, and you need to remove that gate&#39;s rules instead. For example, if you don&#39;t need anymore to explicitly enable or disable a flag for an actor, and the default state should be used instead, you&#39;ll want to cleare the gate. It&#39;s also possible to clear the entire flag, by not passing any option. Options for_actor: an_actor - used to clear the flag for a specific term only. The value can be any term that implements the Actor protocol. for_group: a_group_name - used to clear the flag for a specific group only. The value should be a binary or an atom (It&#39;s internally converted to a binary and it&#39;s stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) boolean: true - used to clear the boolean gate. for_percentage: true - used to clear any percentage gate. Examples iex&gt; alias FunWithFlags.TestUser, as: User iex&gt; harry = %User{id: 1, name: &quot;Harry Potter&quot;, groups: [&quot;wizards&quot;, &quot;gryffindor&quot;]} iex&gt; hagrid = %User{id: 2, name: &quot;Rubeus Hagrid&quot;, groups: [&quot;wizards&quot;, &quot;gamekeeper&quot;]} iex&gt; dudley = %User{id: 3, name: &quot;Dudley Dursley&quot;, groups: [&quot;muggles&quot;]} iex&gt; FunWithFlags.disable(:wands) iex&gt; FunWithFlags.enable(:wands, for_group: &quot;wizards&quot;) iex&gt; FunWithFlags.disable(:wands, for_actor: hagrid) iex&gt; iex&gt; FunWithFlags.enabled?(:wands) false iex&gt; FunWithFlags.enabled?(:wands, for: harry) true iex&gt; FunWithFlags.enabled?(:wands, for: hagrid) false iex&gt; FunWithFlags.enabled?(:wands, for: dudley) false iex&gt; iex&gt; FunWithFlags.clear(:wands, for_actor: hagrid) :ok iex&gt; FunWithFlags.enabled?(:wands, for: hagrid) true iex&gt; iex&gt; FunWithFlags.clear(:wands) :ok iex&gt; FunWithFlags.enabled?(:wands) false iex&gt; FunWithFlags.enabled?(:wands, for: harry) false iex&gt; FunWithFlags.enabled?(:wands, for: hagrid) false iex&gt; FunWithFlags.enabled?(:wands, for: dudley) false","ref":"FunWithFlags.html#clear/2","title":"FunWithFlags.clear/2","type":"function"},{"doc":"Disables a feature flag. Options :for_actor - used to disable the flag for a specific term only. The value can be any term that implements the Actor protocol. :for_group - used to disable the flag for a specific group only. The value should be a binary or an atom (It&#39;s internally converted to a binary and it&#39;s stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) :for_percentage_of - used to disable the flag for a percentage of time or actors, expressed as {:time, float} or {:actors, float}, where float is in the range 0.0 &lt; x &lt; 1.0. Examples Disable globally iex&gt; FunWithFlags.enable(:random_koala_gifs) iex&gt; FunWithFlags.enabled?(:random_koala_gifs) true iex&gt; FunWithFlags.disable(:random_koala_gifs) {:ok, false} iex&gt; FunWithFlags.enabled?(:random_koala_gifs) false Disable for an actor iex&gt; FunWithFlags.enable(:spider_sense) {:ok, true} iex&gt; villain = %{name: &quot;Venom&quot;} iex&gt; FunWithFlags.disable(:spider_sense, for_actor: villain) {:ok, false} iex&gt; FunWithFlags.enabled?(:spider_sense) true iex&gt; FunWithFlags.enabled?(:spider_sense, for: villain) false Disable for a group This example relies on the reference implementation used in the tests. iex&gt; alias FunWithFlags.TestUser, as: User iex&gt; harry = %User{name: &quot;Harry Potter&quot;, groups: [&quot;wizards&quot;, &quot;gryffindor&quot;]} iex&gt; dudley = %User{name: &quot;Dudley Dursley&quot;, groups: [&quot;muggles&quot;]} iex&gt; FunWithFlags.enable(:hogwarts) {:ok, true} iex&gt; FunWithFlags.disable(:hogwarts, for_group: &quot;muggles&quot;) {:ok, false} iex&gt; FunWithFlags.enabled?(:hogwarts) true iex&gt; FunWithFlags.enabled?(:hogwarts, for: harry) true iex&gt; FunWithFlags.enabled?(:hogwarts, for: dudley) false Disable for a percentage of the time iex&gt; FunWithFlags.clear(:random_glitch) :ok iex&gt; FunWithFlags.disable(:random_glitch, for_percentage_of: {:time, 0.999999999}) {:ok, false} iex&gt; FunWithFlags.enabled?(:random_glitch) false iex&gt; FunWithFlags.disable(:random_glitch, for_percentage_of: {:time, 0.000000001}) {:ok, false} iex&gt; FunWithFlags.enabled?(:random_glitch) true Disable for a percentage of the actors iex&gt; FunWithFlags.disable(:new_ui, for_percentage_of: {:actors, 0.3}) {:ok, false}","ref":"FunWithFlags.html#disable/2","title":"FunWithFlags.disable/2","type":"function"},{"doc":"Enables a feature flag. Options :for_actor - used to enable the flag for a specific term only. The value can be any term that implements the Actor protocol. :for_group - used to enable the flag for a specific group only. The value should be a binary or an atom (It&#39;s internally converted to a binary and it&#39;s stored and retrieved as a binary. Atoms are supported for retro-compatibility with versions &lt;= 0.9) :for_percentage_of - used to enable the flag for a percentage of time or actors, expressed as {:time, float} or {:actors, float}, where float is in the range 0.0 &lt; x &lt; 1.0. Examples Enable globally iex&gt; FunWithFlags.enabled?(:super_shrink_ray) false iex&gt; FunWithFlags.enable(:super_shrink_ray) {:ok, true} iex&gt; FunWithFlags.enabled?(:super_shrink_ray) true Enable for an actor iex&gt; FunWithFlags.disable(:warp_drive) {:ok, false} iex&gt; FunWithFlags.enable(:warp_drive, for_actor: &quot;Scotty&quot;) {:ok, true} iex&gt; FunWithFlags.enabled?(:warp_drive) false iex&gt; FunWithFlags.enabled?(:warp_drive, for: &quot;Scotty&quot;) true Enable for a group This example relies on the reference implementation used in the tests. iex&gt; alias FunWithFlags.TestUser, as: User iex&gt; marty = %User{name: &quot;Marty McFly&quot;, groups: [&quot;students&quot;, &quot;time_travelers&quot;]} iex&gt; doc = %User{name: &quot;Emmet Brown&quot;, groups: [&quot;scientists&quot;, &quot;time_travelers&quot;]} iex&gt; buford = %User{name: &quot;Buford Tannen&quot;, groups: [&quot;gunmen&quot;, &quot;bandits&quot;]} iex&gt; FunWithFlags.enable(:delorean, for_group: &quot;time_travelers&quot;) {:ok, true} iex&gt; FunWithFlags.enabled?(:delorean) false iex&gt; FunWithFlags.enabled?(:delorean, for: buford) false iex&gt; FunWithFlags.enabled?(:delorean, for: marty) true iex&gt; FunWithFlags.enabled?(:delorean, for: doc) true Enable for a percentage of the time iex&gt; FunWithFlags.disable(:random_glitch) iex&gt; FunWithFlags.enable(:random_glitch, for_percentage_of: {:time, 0.999999999}) iex&gt; FunWithFlags.enabled?(:random_glitch) true iex&gt; FunWithFlags.enable(:random_glitch, for_percentage_of: {:time, 0.000000001}) iex&gt; FunWithFlags.enabled?(:random_glitch) false Enable for a percentage of the actors This example is based on the fact that the actor score for the actor-flag pair marty + :new_ui is lower than 50%, and for the buford + :new_ui is higher. iex&gt; FunWithFlags.disable(:new_ui) iex&gt; FunWithFlags.enable(:new_ui, for_percentage_of: {:actors, 0.5}) iex&gt; FunWithFlags.enabled?(:new_ui) false iex&gt; alias FunWithFlags.TestUser, as: User iex&gt; marty = %User{id: 42, name: &quot;Marty McFly&quot;} iex&gt; buford = %User{id: 2, name: &quot;Buford Tannen&quot;} iex&gt; FunWithFlags.enabled?(:new_ui, for: marty) true iex&gt; FunWithFlags.enabled?(:new_ui, for: buford) false","ref":"FunWithFlags.html#enable/2","title":"FunWithFlags.enable/2","type":"function"},{"doc":"Checks if a flag is enabled. It can be invoked with just the flag name, as an atom, to check the general staus of a flag (i.e. the boolean gate). Options :for - used to provide a term for which the flag could have a specific value. The passed term should implement the Actor or Group protocol, or both. Examples This example relies on the reference implementation used in the tests. iex&gt; alias FunWithFlags.TestUser, as: User iex&gt; harry = %User{id: 1, name: &quot;Harry Potter&quot;, groups: [&quot;wizards&quot;, &quot;gryffindor&quot;]} iex&gt; FunWithFlags.disable(:elder_wand) iex&gt; FunWithFlags.enable(:elder_wand, for_actor: harry) iex&gt; FunWithFlags.enabled?(:elder_wand) false iex&gt; FunWithFlags.enabled?(:elder_wand, for: harry) true iex&gt; voldemort = %User{id: 7, name: &quot;Tom Riddle&quot;, groups: [&quot;wizards&quot;, &quot;slytherin&quot;]} iex&gt; FunWithFlags.enabled?(:elder_wand, for: voldemort) false iex&gt; filch = %User{id: 88, name: &quot;Argus Filch&quot;, groups: [&quot;staff&quot;]} iex&gt; FunWithFlags.enable(:magic_wands, for_group: &quot;wizards&quot;) iex&gt; FunWithFlags.enabled?(:magic_wands, for: harry) true iex&gt; FunWithFlags.enabled?(:magic_wands, for: voldemort) true iex&gt; FunWithFlags.enabled?(:magic_wands, for: filch) false","ref":"FunWithFlags.html#enabled?/2","title":"FunWithFlags.enabled?/2","type":"function"},{"doc":"Returns a FunWithFlags.Flag struct for the given name, or nil if no flag is found. Useful for debugging.","ref":"FunWithFlags.html#get_flag/1","title":"FunWithFlags.get_flag/1","type":"function"},{"doc":"","ref":"FunWithFlags.html#t:options/0","title":"FunWithFlags.options/0","type":"type"},{"doc":"Implement this protocol to provide actors. Actor gates allows you to enable or disable a flag for one or more entities. For example, in web applications it&#39;s common to use a %User{} struct or equivalent as an actor, or perhaps the data used to represent the current country for an HTTP request. This can be useful to showcase a work-in-progress feature to someone, to gradually rollout a functionality by country, or to dynamically disable some features in some contexts (e.g. a deploy introduces a critical error that only happens in one specific country). Actor gates take precendence over the others, both when they&#39;re enabled and when they&#39;re disabled. They can be considered as toggle overrides. In order to be used as an actor, an entity must implement the FunWithFlags.Actor protocol. This can be implemented for custom structs or literally any other type. Examples This protocol is typically implemented for some application structure. defmodule MyApp.User do defstruct [:id, :name] end defimpl FunWithFlags.Actor, for: MyApp.User do def id(%{id: id}) do &quot;user:\#{id}&quot; end end bruce = %User{id: 1, name: &quot;Bruce&quot;} alfred = %User{id: 2, name: &quot;Alfred&quot;} FunWithFlags.Actor.id(bruce) &quot;user:1&quot; FunWithFlags.Actor.id(alfred) &quot;user:2&quot; FunWithFlags.enable(:batmobile, for_actor: bruce) but it can also be implemented for the builtin types: defimpl FunWithFlags.Actor, for: Map do def id(%{actor_id: actor_id}) do &quot;map:\#{actor_id}&quot; end def id(map) do map |&gt; inspect() |&gt; (&amp;:crypto.hash(:md5, &amp;1)).() |&gt; Base.encode16 |&gt; (&amp;&quot;map:\#{&amp;1}&quot;).() end end defimpl FunWithFlags.Actor, for: BitString do def id(str) do &quot;string:\#{str}&quot; end end FunWithFlags.Actor.id(%{actor_id: &quot;bar&quot;}) &quot;map:bar&quot; FunWithFlags.Actor.id(%{foo: &quot;bar&quot;}) &quot;map:E0BB5BA6873E3AC34B0B6928190C1F2B&quot; FunWithFlags.Actor.id(&quot;foobar&quot;) &quot;string:foobar&quot; FunWithFlags.disable(:foobar, for_actor: %{actor_id: &quot;just a map&quot;}) FunWithFlags.enable(:foobar, for_actor: &quot;just a string&quot;) Actor identifiers must be globally unique binaries. Since supporting multiple kinds of actors is a common requirement, all the examples use the common technique of namespacing the IDs: defimpl FunWithFlags.Actor, for: MyApp.User do def id(user) do &quot;user:\#{user.id}&quot; end end defimpl FunWithFlags.Actor, for: MyApp.Country do def id(country) do &quot;country:\#{country.iso3166}&quot; end end","ref":"FunWithFlags.Actor.html","title":"FunWithFlags.Actor","type":"protocol"},{"doc":"Should return a globally unique binary. Example iex&gt; FunWithFlags.Actor.id(%FunWithFlags.TestUser{id: 313}) &quot;user:313&quot;","ref":"FunWithFlags.Actor.html#id/1","title":"FunWithFlags.Actor.id/1","type":"function"},{"doc":"","ref":"FunWithFlags.Actor.html#t:t/0","title":"FunWithFlags.Actor.t/0","type":"type"},{"doc":"Represents a feature flag. This module is not meant to be used directly.","ref":"FunWithFlags.Flag.html","title":"FunWithFlags.Flag","type":"module"},{"doc":"","ref":"FunWithFlags.Flag.html#t:t/0","title":"FunWithFlags.Flag.t/0","type":"type"},{"doc":"Represents a feature flag gate, that is one of several conditions attached to a feature flag. This module is not meant to be used directly.","ref":"FunWithFlags.Gate.html","title":"FunWithFlags.Gate","type":"module"},{"doc":"","ref":"FunWithFlags.Gate.html#t:t/0","title":"FunWithFlags.Gate.t/0","type":"type"},{"doc":"Implement this protocol to provide groups. Group gates are similar to actor gates, but they apply to a category of entities rather than specific ones. They can be toggled on or off for the name of the group (as an atom) instead of a specific term. Group gates take precendence over boolean gates but are overridden by actor gates. The semantics to determine which entities belong to which groups are application specific. Entities could have an explicit list of groups they belong to, or the groups could be abstract and inferred from some other attribute. For example, an :employee group could comprise all %User{} structs with an email address matching the company domain, or an :admin group could be made of all users with %User{admin: true}. In order to be affected by a group gate, an entity should implement the FunWithFlags.Group protocol. The protocol automatically falls back to a default Any implementation, which states that any entity belongs to no group at all. This makes it possible to safely use &quot;normal&quot; actors when querying group gates, and to implement the protocol only for structs and types for which it matters. The protocol can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [:email, admin: false, groups: []] end defimpl FunWithFlags.Group, for: MyApp.User do def in?(%{email: email}, :employee), do: Regex.match?(~r/@mycompany.com$/, email) def in?(%{admin: is_admin}, :admin), do: !!is_admin def in?(%{groups: list}, group_name), do: group_name in list end elisabeth = %User{email: &quot;elisabeth@mycompany.com&quot;, admin: true, groups: [:engineering, :product]} FunWithFlags.Group.in?(elisabeth, :employee) true FunWithFlags.Group.in?(elisabeth, :admin) true FunWithFlags.Group.in?(elisabeth, :engineering) true FunWithFlags.Group.in?(elisabeth, :marketing) false defimpl FunWithFlags.Group, for: Map do def in?(%{group: group_name}, group_name), do: true def in?(_, _), do: false end FunWithFlags.Group.in?(%{group: :dumb_tests}, :dumb_tests) true With the protocol implemented, actors can be used with the library functions: FunWithFlags.disable(:database_access) FunWithFlags.enable(:database_access, for_group: :engineering)","ref":"FunWithFlags.Group.html","title":"FunWithFlags.Group","type":"protocol"},{"doc":"Should return a boolean. The default implementation will always return false for any argument. Example iex&gt; user = %{name: &quot;bolo&quot;, group: &quot;staff&quot;} iex&gt; FunWithFlags.Group.in?(data, &quot;staff&quot;) true iex&gt; FunWithFlags.Group.in?(data, &quot;superusers&quot;) false","ref":"FunWithFlags.Group.html#in?/2","title":"FunWithFlags.Group.in?/2","type":"function"},{"doc":"","ref":"FunWithFlags.Group.html#t:t/0","title":"FunWithFlags.Group.t/0","type":"type"},{"doc":"A behaviour module for implementing persistence adapters. The package ships with peristence adapters for Redis and Ecto, but you can provide your own adapters by adopting this behaviour.","ref":"FunWithFlags.Store.Persistent.html","title":"FunWithFlags.Store.Persistent","type":"behaviour"},{"doc":"Retrieves all the names of the persisted flags.","ref":"FunWithFlags.Store.Persistent.html#c:all_flag_names/0","title":"FunWithFlags.Store.Persistent.all_flag_names/0","type":"callback"},{"doc":"Retrieves all the persisted flags.","ref":"FunWithFlags.Store.Persistent.html#c:all_flags/0","title":"FunWithFlags.Store.Persistent.all_flags/0","type":"callback"},{"doc":"Deletes an entire flag, identified by name.","ref":"FunWithFlags.Store.Persistent.html#c:delete/1","title":"FunWithFlags.Store.Persistent.delete/1","type":"callback"},{"doc":"Deletes a gate from a flag, identified by name.","ref":"FunWithFlags.Store.Persistent.html#c:delete/2","title":"FunWithFlags.Store.Persistent.delete/2","type":"callback"},{"doc":"Retrieves a flag by name.","ref":"FunWithFlags.Store.Persistent.html#c:get/1","title":"FunWithFlags.Store.Persistent.get/1","type":"callback"},{"doc":"Persists a gate for a flag, identified by name.","ref":"FunWithFlags.Store.Persistent.html#c:put/2","title":"FunWithFlags.Store.Persistent.put/2","type":"callback"},{"doc":"A persistent adapter should return either a child specification if it needs any process to be started and supervised, or nil if it does not. For example, the builtin Redis persistence adapter implements this function by delegating to Redix.child_spec/1 because it needs the Redix processes to work. On the other hand, the builtin Ecto adapter implements this function by returning nil, because the Ecto repo is provided to this package by the host application, and it&#39;s assumed that the Ecto process tree is started and supervised somewhere else. This custom worker_spec/0 function is used instead of the typical child_spec/1 function because this function can return nil if the adapter doesn&#39;t need to be supervised, whereas child_spec/1 must return a valid child spec map.","ref":"FunWithFlags.Store.Persistent.html#c:worker_spec/0","title":"FunWithFlags.Store.Persistent.worker_spec/0","type":"callback"},{"doc":"","ref":"FunWithFlags.Gate.InvalidGroupNameError.html","title":"FunWithFlags.Gate.InvalidGroupNameError","type":"exception"},{"doc":"","ref":"FunWithFlags.Gate.InvalidTargetError.html","title":"FunWithFlags.Gate.InvalidTargetError","type":"exception"},{"doc":"FunWithFlags FunWithFlags, the Elixir feature flag library. If you&#39;re reading this on the GitHub repo, keep in mind that this readme refers to the master branch. For the latest version released on Hex, please check the readme published with the docs. FunWithFlags is an OTP application that provides a 2-level storage to save and retrieve feature flags, an Elixir API to toggle and query them, and a web dashboard as control panel. It stores flag information in Redis or a relational DB (PostgreSQL or MySQL, with Ecto) for persistence and syncronization across different nodes, but it also maintains a local cache in an ETS table for fast lookups. When flags are added or toggled on a node, the other nodes are notified via PubSub and reload their local ETS caches.","ref":"readme.html","title":"FunWithFlags","type":"extras"},{"doc":"What&#39;s a Feature Flag Usage Gate Priority and Interactions Boolean Gate Actor Gate Group Gate Percentage of Time Gate Percentage of Actors Gate Clearing a Feature Flag&#39;s Rules Web Dashboard Origin So, caching, huh? To Do Configuration Persistence Adapters PubSub Adapters Installation Extensibility Custom Persistence Adapters Testing Common Issues","ref":"readme.html#content","title":"FunWithFlags - Content","type":"extras"},{"doc":"Feature flags, or feature toggles, are boolean values associated to a name. They should be used to control whether some application feature is enabled or disabled, and they are meant to be modified at runtime while an application is running. This is usually done by the people who control the application. In their simplest form, flags can be toggled on and off globally. More advanced rules or &quot;gates&quot; allow a fine grained control over their status. For example, it&#39;s possible to toggle a flag on and off for specific entities or for groups. The goal is to have more granular and precise control over what is made available to which users, and when. A common use case, in web applications, is to enable a functionality without the need to deploy or restart the server, or to enable it only for internal users to test it before rolling it out to everyone. Another scenario is the ability to quickly disable a functionality if it&#39;s causing problems. They can also be used to implement a simple authorization system, for example to an admin area.","ref":"readme.html#whats-a-feature-flag","title":"FunWithFlags - What&#39;s a Feature Flag?","type":"extras"},{"doc":"FunWithFlags has a simple API to query and toggle feature flags. Most of the time, you&#39;ll call FunWithFlags.enabled?/2 with the name of the flag and optional arguments. Different kinds of toggle gates are supported: Boolean: globally on and off. Actors: on or off for specific structs or data. The FunWithFlags.Actor protocol can be implemented for types and structs that should have specific rules. For example, in web applications it&#39;s common to use a %User{} struct or equivalent as an actor, or perhaps the current country of the request. Groups: on or off for structs or data that belong to a category or satisfy a condition. The FunWithFlags.Group protocol can be implemented for types and structs that belong to groups for which a feature flag can be enabled or disabled. For example, one could implement the protocol for a %User{} struct to identify administrators. %-of-Time: globally on for a percentage of the time. It ignores actors and groups. Mutually exclusive with the %-of-actors gate. %-of-Actors: globally on for a percentage of the actors. It only applies when the flag is checked with a specific actor and is ignored when the flag is checked without actor arguments. Mutually exclusive with the %-of-time gate. Boolean, Actor and Group gates can express either an enabled or disabled state. The percentage gates can only express an enabled state, as disabling something for a percentage of time or actors is logically equivalent to enabling it for the complementary percentage. Gate Priority and Interactions The priority order is from most to least specific: Actors &gt; Groups &gt; Boolean &gt; Percentage, and it applies to both enabled and disabled gates. For example, a disabled group gate takes precendence over an enabled boolean (global) gate for the entities in the group, and a further enabled actor gate overrides the disabled group gate for a specific entity. When an entity belongs to multiple groups with conflicting toggle status, the disabled group gates have precedence over the enabled ones. The percentage gates are checked last, if present, and they&#39;re only checked if no other gate is enabled. As another example, a flag can have a disabled boolean gate and a 50% enabled %-of-actors gate. When the flag is checked with an actor, it has a (deterministic, consistent and repeatable) 50% chance to be enabled, but when checked without an actor argument it will always be disabled. If we add to the flag a disabled actor gate and an enabled group gate, the flag will be always disabled for the actor, always enabled for any other actor matching the group, have a 50% change to be enabled for any other actor, and always be disabled when checked without actor arguments. If, then, we replace the 50%-of-actors gate with a 50%-of-time gate, the flag will be always disabled for the actor, always enabled for any other actor matching the group, and have a 50% chance to be enabled for any other actor or when checked without an actor argument. Boolean Gate The boolean gate is the simplest one. It&#39;s either enabled or disabled, globally. It&#39;s also the gate with the second lowest priority (it can mask the percentage gates). If a flag is undefined, it defaults to be globally disabled. FunWithFlags.enabled?(:cool_new_feature) false {:ok, true} = FunWithFlags.enable(:cool_new_feature) FunWithFlags.enabled?(:cool_new_feature) true {:ok, false} = FunWithFlags.disable(:cool_new_feature) FunWithFlags.enabled?(:cool_new_feature) false Actor Gate This allows you to enable or disable a flag for one or more entities. For example, in web applications it&#39;s common to use a %User{} struct or equivalent as an actor, or perhaps the data used to represent the current country for an HTTP request. This can be useful to showcase a work-in-progress feature to someone, to gradually rollout a functionality by country, or to dynamically disable some features in some contexts. Actor gates take precendence over the others, both when they&#39;re enabled and when they&#39;re disabled. They can be considered as toggle overrides. In order to be used as an actor, an entity must implement the FunWithFlags.Actor protocol. This can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [:id, :name] end defimpl FunWithFlags.Actor, for: MyApp.User do def id(%{id: id}) do &quot;user:\#{id}&quot; end end bruce = %MyApp.User{id: 1, name: &quot;Bruce&quot;} alfred = %MyApp.User{id: 2, name: &quot;Alfred&quot;} FunWithFlags.Actor.id(bruce) &quot;user:1&quot; FunWithFlags.Actor.id(alfred) &quot;user:2&quot; defimpl FunWithFlags.Actor, for: Map do def id(%{actor_id: actor_id}) do &quot;map:\#{actor_id}&quot; end def id(map) do map |&gt; inspect() |&gt; (&amp;:crypto.hash(:md5, &amp;1)).() |&gt; Base.encode16 |&gt; (&amp;&quot;map:\#{&amp;1}&quot;).() end end FunWithFlags.Actor.id(%{actor_id: &quot;bar&quot;}) &quot;map:bar&quot; FunWithFlags.Actor.id(%{foo: &quot;bar&quot;}) &quot;map:E0BB5BA6873E3AC34B0B6928190C1F2B&quot; With the protocol implemented, actors can be used with the library functions: {:ok, true} = FunWithFlags.enable(:restful_nights) {:ok, false} = FunWithFlags.disable(:restful_nights, for_actor: bruce) {:ok, true} = FunWithFlags.enable(:batmobile, for_actor: bruce) FunWithFlags.enabled?(:restful_nights) true FunWithFlags.enabled?(:batmobile) false FunWithFlags.enabled?(:restful_nights, for: alfred) true FunWithFlags.enabled?(:batmobile, for: alfred) false FunWithFlags.enabled?(:restful_nights, for: bruce) false FunWithFlags.enabled?(:batmobile, for: bruce) true Actor identifiers must be globally unique binaries. Since supporting multiple kinds of actors is a common requirement, all the examples use the common technique of namespacing the IDs: defimpl FunWithFlags.Actor, for: MyApp.User do def id(user) do &quot;user:\#{user.id}&quot; end end defimpl FunWithFlags.Actor, for: MyApp.Country do def id(country) do &quot;country:\#{country.iso3166}&quot; end end Group Gate Group gates are similar to actor gates, but they apply to a category of entities rather than specific ones. They can be toggled on or off for the name of the group instead of a specific term. Group gates take precendence over boolean gates but are overridden by actor gates. Group names can be binaries or atoms. Atoms are supported for retro-compatibility with versions &lt;= 0.9 and binaries are therefore preferred. In fact, atoms are internally converted to binaries and are then stored and later retrieved as binaries. The semantics to determine which entities belong to which groups are application specific. Entities could have an explicit list of groups they belong to, or the groups could be abstract and inferred from some other attribute. For example, an :employee group could comprise all %User{} structs with an email address matching the company domain, or an :admin group could be made of all users with %User{admin: true}. In order to be affected by a group gate, an entity should implement the FunWithFlags.Group protocol. The protocol automatically falls back to a default Any implementation, which states that any entity belongs to no group at all. This makes it possible to safely use &quot;normal&quot; actors when querying group gates, and to implement the protocol only for structs and types for which it matters. The protocol can be implemented for custom structs or literally any other type. defmodule MyApp.User do defstruct [:email, admin: false, groups: []] end defimpl FunWithFlags.Group, for: MyApp.User do def in?(%{email: email}, &quot;employee&quot;), do: Regex.match?(~r/@mycompany.com$/, email) def in?(%{admin: is_admin}, &quot;admin&quot;), do: !!is_admin def in?(%{groups: list}, group_name), do: group_name in list end elisabeth = %MyApp.User{email: &quot;elisabeth@mycompany.com&quot;, admin: true, groups: [&quot;engineering&quot;, &quot;product&quot;]} FunWithFlags.Group.in?(elisabeth, &quot;employee&quot;) true FunWithFlags.Group.in?(elisabeth, &quot;admin&quot;) true FunWithFlags.Group.in?(elisabeth, &quot;engineering&quot;) true FunWithFlags.Group.in?(elisabeth, &quot;marketing&quot;) false defimpl FunWithFlags.Group, for: Map do def in?(%{group: group_name}, group_name), do: true def in?(_, _), do: false end FunWithFlags.Group.in?(%{group: &quot;dumb_tests&quot;}, &quot;dumb_tests&quot;) true With the protocol implemented, actors can be used with the library functions: FunWithFlags.disable(:database_access) FunWithFlags.enable(:database_access, for_group: &quot;engineering&quot;) FunWithFlags.enabled?(:database_access) false FunWithFlags.enabled?(:database_access, for: elisabeth) true Percentage of Time Gate %-of-time gates are similar to boolean gates, but they allow to enable a flag for a percentage of the time. In practical terms, this means that a percentage of the enabled?() calls for a flag will return true, regardless of the presence of an actor argument. When a %-of-time gate is checked a pseudo-random number is generated and compared with the percentage value of the gate. If the result of the random roll is lower than the gate&#39;s percentage value, the gate is considered enabled. So, at the risk of stating the obvious and for the sake of clarity, a 90% gate is enabled more often than a 10% gate. %-of-time gates are useful to gradually introduce alternative code paths that either have the same effects of the old ones, or don&#39;t have effects visible to the users. This last point is important, because with a %-of-time gate the application will behave differently on a pseudo-random basis. A good use case for %-of-time gates is to safely test the correctedness or performance and load characteristics of an alternative implementation of a functionality. For example: FunWithFlags.clear(:alternative_implementation) FunWithFlags.enable(:alternative_implementation, for_percentage_of: {:time, 0.05}) def foo(bar) do if FunWithFlags.enabled?(:alternative_implementation) do new_foo(bar) else old_foo(bar) end end The %-of-time gate is incompatible and mutually exclusive with the %-of-actors gate, and it replaces it when it gets set. While there are ways to make them work together, it would needlessly overcomplicate the priority rules. Percentage of Actors Gate %-of-actors gates are similar to the %-of-time gates, but instead of using a pseudo-random chance they calculate the actor scores using a deterministic, consistent and repeatable function that factors in the flag name. At a high level: actor |&gt; FunWithFlags.Actor.id() |&gt; sha256_hash(flag_name) |&gt; hash_to_percentage() Since the scores depend on both the actor ID and the flag name, they&#39;re guaranteed to always be the same for each actor-flag combination. At the same time, the same actor will have different scores for different flags, and each flag will have a uniform distribution of scores for all the actors. Just like for the %-of-time gates, an actor&#39;s score is compared with the gate&#39;s percentage value and, if lower, the gate will result enabled. A practical example, based on the FunWithFlags.Actor protocol set up from the previous sections: defmodule MyApp.User do defstruct [:id, :name] end defimpl FunWithFlags.Actor, for: MyApp.User do def id(%{id: id}) do &quot;user:\#{id}&quot; end end frodo = %MyApp.User{id: 1, name: &quot;Frodo Baggins&quot;} sam = %MyApp.User{id: 2, name: &quot;Samwise Gamgee&quot;} pippin = %MyApp.User{id: 3, name: &quot;Peregrin Took&quot;} merry = %MyApp.User{id: 4, name: &quot;Meriadoc Brandybuck&quot;} FunWithFlags.Actor.score(frodo, :pipeweed) 0.8658294677734375 FunWithFlags.Actor.score(sam, :pipeweed) 0.68426513671875 FunWithFlags.Actor.score(pippin, :pipeweed) 0.510528564453125 FunWithFlags.Actor.score(merry, :pipeweed) 0.2617645263671875 {:ok, true} = FunWithFlags.enable(:pipeweed, for_percentage_of: {:actors, 0.60}) FunWithFlags.enabled?(:pipeweed, for: frodo) false FunWithFlags.enabled?(:pipeweed, for: sam) false FunWithFlags.enabled?(:pipeweed, for: pippin) true FunWithFlags.enabled?(:pipeweed, for: merry) true {:ok, true} = FunWithFlags.enable(:pipeweed, for_percentage_of: {:actors, 0.685}) FunWithFlags.enabled?(:pipeweed, for: sam) true FunWithFlags.Actor.score(pippin, :pipeweed) 0.510528564453125 FunWithFlags.Actor.score(pippin, :mushrooms) 0.6050872802734375 FunWithFlags.Actor.score(pippin, :palantir) 0.144073486328125 Once a %-of-actors gate has been defined for a flag, the same actor will always see the same result (unless its actor or group gates are set, or the flag gets globally enabled). Also, this means that as long the percentage value of the gate will increase and never decrease, actors for which the gate has been enabled will always see it enabled. This is ideal to gradually roll out new functionality to users. For example, in a Phoenix application: FunWithFlags.clear(:new_design) FunWithFlags.enable(:new_design, for_percentage_of: {:actors, 0.2}) FunWithFlags.enable(:new_design, for_group: &quot;beta_testers&quot;) defmodule MyPhoenixApp.MyView do use MyPhoenixApp, :view def render(&quot;my_template.html&quot;, assigns) do if FunWithFlags.enabled?(:new_design, for: assigns.user) do render(&quot;new_template.html&quot;, assigns) else render(&quot;old_template.html&quot;, assigns) end end end The %-of-actors gate is incompatible and mutually exclusive with the %-of-time gate, and it replaces it when it gets set. While there are ways to make them work together, it would needlessly overcomplicate the priority rules. Clearing a Feature Flag&#39;s Rules Sometimes enabling or disabling a gate is not what you want, and removing that gate&#39;s rules would be more correct. For example, if you don&#39;t need anymore to explicitly enable or disable a flag for an actor or for a group, and the default state should be used instead, clearing the gate is the right choice. More examples: alias FunWithFlags.TestUser, as: User harry = %User{id: 1, name: &quot;Harry Potter&quot;, groups: [&quot;wizards&quot;, &quot;gryffindor&quot;]} hagrid = %User{id: 2, name: &quot;Rubeus Hagrid&quot;, groups: [&quot;wizards&quot;, &quot;gamekeeper&quot;]} dudley = %User{id: 3, name: &quot;Dudley Dursley&quot;, groups: [&quot;muggles&quot;]} FunWithFlags.disable(:wands) FunWithFlags.enable(:wands, for_group: &quot;wizards&quot;) FunWithFlags.disable(:wands, for_actor: hagrid) FunWithFlags.enabled?(:wands) false FunWithFlags.enabled?(:wands, for: harry) true FunWithFlags.enabled?(:wands, for: hagrid) false FunWithFlags.enabled?(:wands, for: dudley) false FunWithFlags.clear(:wands, for_actor: hagrid) FunWithFlags.enabled?(:wands, for: hagrid) true FunWithFlags.clear(:wands, for_group: &quot;wizards&quot;) FunWithFlags.enabled?(:wands, for: hagrid) false FunWithFlags.enabled?(:wands, for: harry) false FunWithFlags.enable(:magic_powers, for_percentage_of: {:time, 0.0001}) FunWithFlags.clear(:magic_powers, for_percentage: true) For completeness, clearing the boolean gate is also supported. FunWithFlags.enable(:wands) FunWithFlags.enabled?(:wands) true FunWithFlags.enabled?(:wands, for: harry) true FunWithFlags.enabled?(:wands, for: hagrid) false FunWithFlags.enabled?(:wands, for: dudley) true FunWithFlags.clear(:wands, boolean: true) FunWithFlags.enabled?(:wands) false FunWithFlags.enabled?(:wands, for: harry) true FunWithFlags.enabled?(:wands, for: hagrid) false FunWithFlags.enabled?(:wands, for: dudley) false It&#39;s also possible to clear an entire flag. FunWithFlags.clear(:wands) FunWithFlags.enabled?(:wands) false FunWithFlags.enabled?(:wands, for: harry) false FunWithFlags.enabled?(:wands, for: hagrid) false FunWithFlags.enabled?(:wands, for: dudley) false","ref":"readme.html#usage","title":"FunWithFlags - Usage","type":"extras"},{"doc":"An optional extension of this library is FunWithFlags.UI, a web graphical control panel. It&#39;s a Plug, so it can be embedded in a host Phoenix or Plug application or served standalone.","ref":"readme.html#web-dashboard","title":"FunWithFlags - Web Dashboard","type":"extras"},{"doc":"This library is heavily inspired by the flipper Ruby gem. Having used Flipper in production at scale, this project aims to improve in two main areas: Minimize the load on the persistence layer: feature flags are not toggled that often, and there is no need to query Redis or the DB for each check. Be more reliable: it should keep working with the latest cached values even if Redis becomes unavailable, although with the risk of nodes getting out of sync. (if the DB becomes unavailable, feature flags are probably the last of your problems) Just as Elixir and Phoenix are meant to scale better than Ruby on Rails with high levels of traffic and concurrency, FunWithFlags should aim to be more scalable and reliable than Flipper.","ref":"readme.html#origin","title":"FunWithFlags - Origin","type":"extras"},{"doc":"There are only two hard things in Computer Science: cache invalidation and naming things. -- Phil Karlton The reason to add an ETS cache is that, most of the time, feature flags can be considered static values. Doing a round-trip to the DB (Redis, PostgreSQL or MySQL) is expensive in terms of time and in terms of resources, expecially if multiple flags must be checked during a single web request. In the worst cases, the load on the DB can become a cause of concern, a performance bottleneck or the source of a system failure. Often the solution is to memoize the flag values in the context of the web request, but the approach can be extended to the scope of the entire server. This is what FunWithFlags does, as each application node/instance caches the flags in an ETS table. Of course, caching adds a different kind of complexity and there are some pros and cons. When a flag is created or updated the ETS cache on the local node is updated immediately, and the main problem is syncronizing the flag data across the other application nodes that should share the same view of the world. For example, if we have two or more nodes running the application, and on one of them an admin user updates a flag that the others have already cached, or creates a flag that the others have already looked up (and cached as &quot;disabled&quot;), then the other nodes must be notified of the changes. FunWithFlags uses three mechanisms to deal with the problem: Use PubSub to emit change notifications. All nodes subscribe to the same channel and reload flags in the ETS cache when required. If that fails, the cache has a configurable TTL. Reading from redis every few minutes is still better than doing so 30k times per second. If that doesn&#39;t work, it&#39;s possible to disable the cache and just read from the DB all the time. That&#39;s what Flipper does. In terms of performance, very synthetic benchmarks (where the DBs run on the same machine as the Beam code, so with no network hop but sharing the CPU) show that the ETS cache makes querying the FunWithFlags interface between 10 and 20 times faster than going directly to Redis, and between 20 and 40 times faster than going directly to Postgres. The variance depends on the complexity of the flag data to be retrieved.","ref":"readme.html#so-caching-huh","title":"FunWithFlags - So, caching, huh?","type":"extras"},{"doc":"Add some optional randomness to the TTL, so that Redis or the DB don&#39;t get hammered at constant intervals after a server restart.","ref":"readme.html#to-do","title":"FunWithFlags - To Do","type":"extras"},{"doc":"The library can be configured in host applications through Mix and the config.exs file. This example shows some default values: config :fun_with_flags, :cache, enabled: true, ttl: 900 # in seconds # the Redis persistence adapter is the default, no need to set this. config :fun_with_flags, :persistence, [adapter: FunWithFlags.Store.Persistent.Redis] # this can be disabled if you are running on a single node and don&#39;t need to # sync different ETS caches. It won&#39;t have any effect if the cache is disabled. # The Redis PuSub adapter is the default, no need to set this. config :fun_with_flags, :cache_bust_notifications, [enabled: true, adapter: FunWithFlags.Notifications.Redis] # Notifications can also be disabled, which will also remove the Redis/Redix dependency config :fun_with_flags, :cache_bust_notifications, [enabled: false] When using Redis for persistence and/or cache-busting PubSub it is necessary to configure the connection to the Redis instance. These options can be omitted if Redis is not being used. For example, the defaults: # the Redis options will be forwarded to Redix. config :fun_with_flags, :redis, host: &quot;localhost&quot;, port: 6379, database: 0 # a URL string can be used instead config :fun_with_flags, :redis, &quot;redis://locahost:6379/0&quot; # a {:system, name} tuple can be used to read from the environment config :fun_with_flags, :redis, {:system, &quot;REDIS_URL&quot;} Persistence Adapters The library comes with two persistence adapters for the Redix and Ecto libraries, that allow to persist feature flag data in Redis, PostgreSQL or MySQL. In order to use any of them, you must declare the correct optional dependency in the Mixfile (see the installation instructions, below). The Redis adapter is the default and there is no need to explicitly declare it. All it needs is the Redis connection configuration. In order to use the Ecto adapter, an Ecto repo must be provided in the configuration. FunWithFlags expects the Ecto repo to be initialized by the host application, which also needs to start and supervise any required processes. If using Phoenix this is managed automatically by the framework, and it&#39;s fine to use the same repo used by the rest of the application. Only PostgreSQL (via postgrex) and MySQL (via mariaex or myxql) are supported at the moment. Support for other RDBMSs might come in the future. To configure the Ecto adapter: # Normal Phoenix and Ecto configuration. # The repo can either use the Postgres or MySQL adapter. config :my_app, ecto_repos: [MyApp.Repo] config :my_app, MyApp.Repo, username: &quot;my_db_user&quot;, password: &quot;my secret db password&quot;, database: &quot;my_app_dev&quot;, hostname: &quot;localhost&quot;, pool_size: 10 # FunWithFlags configuration. config :fun_with_flags, :persistence, adapter: FunWithFlags.Store.Persistent.Ecto, repo: MyApp.Repo, ecto_table_name: &quot;your_table_name&quot; # optional # Default table name is &quot;fun_with_flags_toggles&quot;. It&#39;s also necessary to create the DB table that will hold the feature flag data. To do that, create a new migration in your project and copy the contents of the provided migration file. Then run the migration. PubSub Adapters The library comes with two PubSub adapters for the Redix and Phoenix.PubSub libraries. In order to use any of them, you must declare the correct optional dependency in the Mixfile. (see the installation instructions, below) The Redis PubSub adapter is the default and doesn&#39;t need to be excplicity configured. It can only be used in conjunction with the Redis persistence adapter however, and is not available when using Ecto for persistence. When used, it connects directly to the Redis instance used for persisting the flag data. The Phoenix PubSub adapter uses the high level API of Phoenix.PubSub, which means that under the hood it could use either its PG2 or Redis adapters, and this library doesn&#39;t need to know. It&#39;s provided as a convenient way to leverage distributed Erlang when using FunWithFlags in a Phoenix application, although it can be used independently (without the rest of the Phoenix framework) to add PubSub to Elixir apps running on Erlang clusters.FunWithFlags expects the Phoenix.PubSub process to be started by the host application, and in order to use this adapter the client (name or PID) must be provided in the configuration. For example, in Phoenix (&gt;= 1.5.0) it would be: # normal Phoenix configuration config :my_app, MyApp.Web.Endpoint, pubsub_server: MyApp.PubSub # FunWithFlags configuration config :fun_with_flags, :cache_bust_notifications, enabled: true, adapter: FunWithFlags.Notifications.PhoenixPubSub, client: MyApp.PubSub Or, without Phoenix: # possibly in the application&#39;s supervision tree children = [ {Phoenix.PubSub, [name: :my_pubsub_process_name, adapter: Phoenix.PubSub.PG2]} ] opts = [strategy: :one_for_one, name: MyApp.Supervisor] {:ok, _pid} = Supervisor.start_link(children, opts) # config/config.exs config :fun_with_flags, :cache_bust_notifications, enabled: true, adapter: FunWithFlags.Notifications.PhoenixPubSub, client: :my_pubsub_process_name","ref":"readme.html#configuration","title":"FunWithFlags - Configuration","type":"extras"},{"doc":"The package can be installed by adding fun_with_flags to your list of dependencies in mix.exs. In order to have a small installation footprint, the dependencies for the different adapters are all optional. You must explicitly require the ones you wish to use. def deps do [ {:fun_with_flags, &quot;~&gt; 1.5.1&quot;}, # either: {:redix, &quot;~&gt; 0.9&quot;}, # or: {:ecto_sql, &quot;~&gt; 3.0}&quot;, # optionally, if you don&#39;t want to use Redis&#39; builtin pubsub {:phoenix_pubsub, &quot;~&gt; 2.0&quot;}, ] end Using ecto_sql for persisting the flags also requires an ecto adapter, e.g. postgrex, mariaex or myxql. Please refer to the Ecto documentation for the details. Since FunWithFlags depends on Elixir &gt;= 1.6, there is no need to explicitly declare the application.","ref":"readme.html#installation","title":"FunWithFlags - Installation","type":"extras"},{"doc":"Custom Persistence Adapters This library aims to be extensible and allows users to provide their own persistence layer. This is supported through FunWithFlags.Store.Persistent, a generic persistence behaviour that is adopted by the builtin Redis and Ecto adapters. Custom persistence adapters can adopt the behaviour and then be configured as the persistence module in the Mix config of the user applications. For example, an application can define this module: defmodule MyApp.MyAlternativeFlagStore do @behaviour FunWithFlags.Store.Persistent # implement all the behaviour&#39;s callback end And then configure the library to use it: config :fun_with_flags, :persistence, adapter: MyApp.MyAlternativeFlagStore","ref":"readme.html#extensibility","title":"FunWithFlags - Extensibility","type":"extras"},{"doc":"This library depends on Redis, PostgreSQL and MySQL, and you&#39;ll need them installed and running on your system in order to run the complete test suite. The tests will use the Redis db number 5 and then clean after themselves, but it&#39;s safer to start Redis in a directory where there is no dump.rdb file you care about to avoid issues. The Ecto tests will use the SQL sandbox and all transactions will be automatically rolled back. To setup the test DB for the Ecto persistence tests, run: MIX_ENV=test PERSISTENCE=ecto mix do ecto.create, ecto.migrate # for postgres rm -rf _build/test/lib/fun_with_flags/ MIX_ENV=test PERSISTENCE=ecto RDBMS=mysql mix do ecto.create, ecto.migrate # for mysql Then, to run all the tests: $ mix test.all The test.all task will run the test suite multiple times with different configurations to exercise a matrix of options and adapters. The Mixfile defines a few other helper tasks that allow to run the test suite with some more specific configurations.","ref":"readme.html#testing","title":"FunWithFlags - Testing","type":"extras"},{"doc":"Configuration changes have no effect in MIX_ENV=dev Issue: changing the library settings in the host application&#39;s Mix config file has no effect, or &quot;missing process&quot; exceptions are raised when booting. This should only be an issue in the development environment.Solution: clear the compiled BEAM bytecode with: rm -r _build/dev/lib/fun_with_flags rm -r _build/dev/lib/YOUR_APP_NAME Why does this happen? This library tries to be fast and performant and it optimizes some things at compile-time by conditionally compiling different references to modules. Specifically, if the cache is enabled or disabled the library will use a different module (either Store or SimpleStore). Because these modules are accessed in the hot path, compiling them in improves the performance of the library. While it would be possible to read that config dynamically with Application.get_env/3, that would require a few extra calls that should not really be necessary for data that never changes while the application is running. These calls would be reading the config from the application environment ETS table, plus some function calls to merge it with the defaults and then read the right key. Those extra operations come with some overhead. For this reason, things that are referenced in the hot path of the library are frozen at compile time with module attributes. This effectively minimizes needless inter-process messages, and gives a nice performance boost to busy applications. This should only be a problem in the development environment, where if you change some settings after the initial compilation, Mix will not recompile the dependencies. In the production enviroment this should not be an issue.","ref":"readme.html#common-issues","title":"FunWithFlags - Common Issues","type":"extras"}]